-- Example code for the receiver datapath 
-- It is composed of up to 6 components 
-- It is an strcutural design 

LIBRARY ieee;	
USE  IEEE.STD_LOGIC_1164.all;

ENTITY Receiver_datapath IS  
	PORT(
		Serial_in,CD,CLK	: 	IN 	std_logic;
		LDD					: 	IN 	std_logic;			
		CE_Mod4,CE_Mod8		:	IN 	std_logic;
		Clear_Counters		:	IN 	std_logic;
		s					:	IN	std_logic_vector(1 downto 0);
		RX_out				:	OUT std_logic_vector(7 downto 0);
		EP_Check,Stop_bit	:	OUT std_logic;									
		TC4,TC8,TC10		:	OUT std_logic
	);
END Receiver_datapath;


ARCHITECTURE Structural OF Receiver_datapath IS
-- Components
	COMPONENT shift_register_10bit IS  
		PORT(	
				CD,CLK		: 	IN	std_logic;
				D_in		:	IN	std_logic_vector(9 downto 0);
				S			: 	IN 	std_logic_vector(1 downto 0);
				RSI,LSI		: 	IN 	std_logic;
				Q 	  		:	OUT	STD_LOGIC_VECTOR(9 DOWNTO 0)

		);
	END COMPONENT;


	COMPONENT data_register_8bits IS  
		Port (	
			CLK		: 	IN	STD_LOGIC;
			CD		: 	IN	STD_LOGIC;
		    LD		: 	IN	STD_LOGIC;
			Q		: 	OUT	STD_LOGIC_VECTOR(7 DOWNTO 0);
			D_in	: 	IN STD_LOGIC_VECTOR(7 DOWNTO 0)
		);
	END COMPONENT;


	COMPONENT parity_checker_9bit IS  
		PORT (
			D	:	IN 	STD_LOGIC_VECTOR(8 DOWNTO 0); 				
			Y	: 	OUT  	STD_LOGIC
		);

	END COMPONENT;

	COMPONENT counter_mod4 IS  
		Port (	
			CLK	: IN	STD_LOGIC;
			CD	: IN	STD_LOGIC;
	        CE	: IN	STD_LOGIC;
			TC4	: OUT	STD_LOGIC
		);
	END COMPONENT;

	COMPONENT counter_mod8 IS  
		Port (	
			CLK	: IN	STD_LOGIC;
			CD	: IN	STD_LOGIC;
			CE	: IN	STD_LOGIC;
			TC8	: OUT	STD_LOGIC
		);
	END COMPONENT;

	COMPONENT counter_mod10 IS  
		Port (	
			CLK		: IN	STD_LOGIC;
			CD		: IN	STD_LOGIC;
			CE		: IN	STD_LOGIC;
			TC10	: OUT	STD_LOGIC
		);
	END COMPONENT;

-- Internal wires or signals 
	SIGNAL B		:	STD_LOGIC_VECTOR(7 downto 0);
 	SIGNAL CE_Mod10	:	STD_LOGIC; 
 	SIGNAL EPB		:	STD_LOGIC;  	


BEGIN
-- Instantiation of components
Chip4	: counter_mod4
	PORT MAP (	   	
-- from component name 	=> to signal or port name
		CLK	=> CLK,
		CD 	=> Clear_Counters,
		TC4 => TC4,
		CE	=> CE_Mod4
	);

Chip5	: counter_mod8
	PORT MAP (	   	
-- from component name 	=> to signal or port name
		CLK	=> CLK,
		CD 	=> Clear_Counters,
		TC8	=> CE_Mod10,
		CE	=> CE_Mod8

	);

Chip6 	: counter_mod10
	PORT MAP (	   	
-- from component name 	=> to signal or port name
		CLK		=> CLK,
		CD 		=> Clear_Counters,
		TC10	=> TC10,
		CE		=> CE_Mod10

	);

Chip1	: data_register_8bits
	PORT MAP (	   	
-- from component name 	=> to signal or port name
		D_in	=> B,
		LD 		=> LDD,
		CD 		=> CD,
		CLK		=> CLK,
		Q 		=> Rx_out
	);

Chip2	: shift_register_10bit
	PORT MAP (	   	
-- from component name 	=> to signal or port name
		CLK				=> CLK,
		CD 				=> CD,
		D_in			=> "0000000000",
		Q(7 downto 0)	=> B, 
		Q(8)			=> EPB,
		Q(9)			=> Stop_bit,
		S				=> S,
		RSI				=> Serial_in,
		LSI				=> '0'	
	);

Chip3 	: parity_checker_9bit
	PORT MAP (	   	
-- from component name 	=> to signal or port name
		D (7 downto 0)	=> B,
		D(8) 			=> EPB,
		Y 				=> EP_check	
	);


-- Extra logic in the circuit
TC8 <= CE_mod10;


END Structural;



